@@ -15,7 +15,7 @@
 #
 
 package MongoDB::Connection;
-our $VERSION = '0.40';
+our $VERSION = '0.41';
 
 # ABSTRACT: A connection to a Mongo server
 
@@ -403,8 +403,6 @@ Port to use when connecting to right_host. Defaults to C<27017>.
 =cut
 
 
-sub CLONE_SKIP { 1 }
-
 sub AUTOLOAD {
     my $self = shift @_;
     our $AUTOLOAD;
@@ -460,6 +458,9 @@ sub BUILD {
         $opts->{host} = "mongodb://$_";
 
         $self->_servers->{$_} = MongoDB::Connection->new($opts);
+
+        next unless $self->auto_connect;
+
         # it's okay if we can't connect, so long as someone can
         eval {
             $self->_servers->{$_}->connect;
@@ -471,14 +472,23 @@ sub BUILD {
         }
     }
 
-    # if we still aren't connected to anyone, give up
-    if (!$connected) {
-        die "couldn't connect to any servers listed: ".join(",", @pairs);
-    }
+    my $master;
 
-    my $master = $self->get_master;
-    if ($master == -1) {
-        die "couldn't find master";
+    if ($self->auto_connect) {
+
+        # if we still aren't connected to anyone, give up
+        if (!$connected) {
+            die "couldn't connect to any servers listed: ".join(",", @pairs);
+        }
+
+        $master = $self->get_master;
+        if ($master == -1) {
+            die "couldn't find master";
+        }
+    }
+    else {
+        # no auto-connect so just pick one. if auto-reconnect is set then it will connect as needed
+        ($master) = values %{$self->_servers};
     }
 
     # create a struct that just points to the master's connection
@@ -725,7 +735,7 @@ C<$info> hash will be automatically created for you by L<MongoDB::write_query>.
 =cut
 
 no Any::Moose;
-__PACKAGE__->meta->make_immutable (inline_destructor => 0);
+__PACKAGE__->meta->make_immutable;
 
 1;
 
