@@ -23,9 +23,7 @@
 #include "perl.h"
 #include "XSUB.h"
 
-#ifndef mXPUSHp
-#define mXPUSHp(p,l) STMT_START { EXTEND(sp,1); sv_setpvn(PUSHmort al, (p), (l)); } STMT_END
-#endif
+#include "ppport.h"
 
 #define PERL_MONGO_CALL_BOOT(name)  perl_mongo_call_xs (aTHX_ name, cv, mark)
 
@@ -116,20 +114,29 @@ typedef struct _stackette {
 
 #define EMPTY_STACK 0
 
+// it's safer to leave this signed in case there are any other missing BUF_REMAININGs
 #define BUF_REMAINING (buf->end-buf->pos)
 #define set_type(buf, type) perl_mongo_serialize_byte(buf, (char)type)
 #define perl_mongo_serialize_null(buf) perl_mongo_serialize_byte(buf, (char)0)
 #define perl_mongo_serialize_bool(buf, b) perl_mongo_serialize_byte(buf, (char)b)
 
+extern MGVTBL connection_vtbl, cursor_vtbl;
+extern int perl_mongo_machine_id;
+
+#ifdef WIN32
+// Windows requires initializing the mutex before using it.
+void perl_mongo_mutex_init();
+#endif
+
 void perl_mongo_call_xs (pTHX_ void (*subaddr) (pTHX_ CV *cv), CV *cv, SV **mark);
 SV *perl_mongo_call_reader (SV *self, const char *reader);
-SV *perl_mongo_call_method (SV *self, const char *method, int num, ...);
+SV *perl_mongo_call_method (SV *self, const char *method, I32 flags, int num, ...);
 SV *perl_mongo_call_function (const char *func, int num, ...);
-void perl_mongo_attach_ptr_to_instance (SV *self, void *ptr);
-void *perl_mongo_get_ptr_from_instance (SV *self);
+void perl_mongo_attach_ptr_to_instance (SV *self, void *ptr, MGVTBL *vtbl);
+void *perl_mongo_get_ptr_from_instance (SV *self, MGVTBL *vtbl);
 SV *perl_mongo_construct_instance (const char *klass, ...);
 SV *perl_mongo_construct_instance_va (const char *klass, va_list ap);
-SV *perl_mongo_construct_instance_with_magic (const char *klass, void *ptr, ...);
+SV *perl_mongo_construct_instance_with_magic (const char *klass, void *ptr, MGVTBL *vtbl, ...);
 
 void perl_mongo_make_id(char *id);
 void perl_mongo_make_oid(char* twelve, char *twenty4);
@@ -142,10 +149,10 @@ int perl_mongo_resize_buf (buffer*, int);
 void perl_mongo_serialize_key(buffer *buf, const char *str, int is_insert);
 void perl_mongo_serialize_size(char*, buffer*);
 void perl_mongo_serialize_double(buffer*, double);
-void perl_mongo_serialize_string(buffer*, const char*, int);
+void perl_mongo_serialize_string(buffer*, const char*, unsigned int);
 void perl_mongo_serialize_long(buffer*, int64_t);
 void perl_mongo_serialize_int(buffer*, int);
 void perl_mongo_serialize_byte(buffer*, char);
-void perl_mongo_serialize_bytes(buffer*, const char*, int);
+void perl_mongo_serialize_bytes(buffer*, const char*, unsigned int);
 
 #endif
